@@ -3,16 +3,16 @@
 import math
 import mimetypes
 
-
 import numpy as np
 import cv2
 
-
 CRED = '\033[0;31m'
 CCYAN = '\033[0;36m'
 CGREEN = '\033[0;32m'
 CRESET = '\033[0m'
 
+FONT_TYPE = [cv2.FONT_HERSHEY_COMPLEX_SMALL, cv2.FONT_HERSHEY_SIMPLEX, cv2.FONT_HERSHEY_TRIPLEX]
+
 
 def gamma(img, g):  # @public
     lookUpTable = np.empty((1, 256), np.uint8)
@@ -74,8 +74,8 @@ def concat_bhwc_image(ts):
             row = col
         if i == bsize - 1:
             break
-    
-    return row #np.array(row * 255, dtype=np.uint8)
+
+    return row  #np.array(row * 255, dtype=np.uint8)
 
 
 def rgb2bgr(img):  # @public
@@ -83,7 +83,7 @@ def rgb2bgr(img):  # @public
         raise "src image channel must be 3(RGB)"
     if img.dtype != np.uint8:
         raise "expected dtype is uint8."
-    return cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
+    return img[..., ::-1]
 
 
 def draw_image_alpha(img, img_rgba, sx, sy):  # @public
@@ -91,25 +91,20 @@ def draw_image_alpha(img, img_rgba, sx, sy):  # @public
     return img
 
 
-def draw_footer(img, message, color=(255, 200, 55), bg=(55, 55, 55)):  # @public
+def draw_footer(img, message, color=(255, 200, 55), bg=(55, 55, 55), font_scale=1, font_type=0):  # @public
     h, w, c = img.shape
     cv2.rectangle(img, (0, h), (w, h - 20), bg, -1)
-    fontScale = 1
-    cv2.putText(img, message, (5, h - 4), cv2.FONT_HERSHEY_COMPLEX_SMALL,
-                fontScale, color, 1, lineType=cv2.LINE_AA)
+    cv2.putText(img, message, (5, h - 4), FONT_TYPE[font_type], font_scale, color, 1, lineType=cv2.LINE_AA)
 
 
-def draw_title(img, message, color=(255, 200, 55), bg=(55, 55, 55)):  # @public
+def draw_title(img, message, color=(255, 200, 55), bg=(55, 55, 55), font_scale=1, font_type=0):  # @public
     h, w, c = img.shape
     cv2.rectangle(img, (0, 0), (w, 20), bg, -1)
-    fontScale = 1
-    cv2.putText(img, message, (5, 17), cv2.FONT_HERSHEY_COMPLEX_SMALL,
-                fontScale, color, 1, lineType=cv2.LINE_AA)
+    cv2.putText(img, message, (5, 17), FONT_TYPE[font_type], font_scale, color, 1, lineType=cv2.LINE_AA)
 
 
-def draw_text(img, message, x=5, y=17, color=(255, 200, 55), fontScale=1):  # @public
-    cv2.putText(img, message, (x, y), cv2.FONT_HERSHEY_COMPLEX_SMALL,
-                fontScale, color, 1, lineType=cv2.LINE_AA)
+def draw_text(img, message, x=5, y=17, color=(255, 200, 55), font_scale=1, font_type=0):  # @public
+    cv2.putText(img, message, (x, y), FONT_TYPE[font_type], font_scale, color, 1, lineType=cv2.LINE_AA)
 
 
 def is_image_ext(f):  # @public
@@ -129,9 +124,17 @@ def is_image_ext(f):  # @public
 
 def opencv_decoder(data):
     b = data
-    nb = np.asarray(b, dtype=np.uint8)
+    if isinstance(data, np.ndarray):
+        nb = data
+    elif isinstance(data, bytes):
+        nb = np.frombuffer(b, dtype=np.uint8)
+    elif isinstance(data, bytearray):
+        nb = np.frombuffer(b, dtype=np.uint8)
+    else:
+        raise Exception("Arg type must be bytes/bytearray/ndarray. Invalid arg:" + type(data))
     data = cv2.imdecode(nb, cv2.IMREAD_COLOR)
-    data = cv2.cvtColor(data, cv2.COLOR_BGR2RGB)
+    data = data[..., ::-1]
+
     return data
 
 
@@ -139,10 +142,9 @@ def opencv_encoder(data, **kargs):
     quality = 90
     if "quality" in kargs:
         quality = kargs["quality"]
-    data = cv2.cvtColor(data, cv2.COLOR_BGR2RGB)
-    check, data = cv2.imencode(
-        ".jpg", data, [int(cv2.IMWRITE_JPEG_QUALITY), quality])  # quality 1-100
-    if check == False:
+    data = data[..., ::-1]
+    check, data = cv2.imencode(".jpg", data, [int(cv2.IMWRITE_JPEG_QUALITY), quality])  # quality 1-100
+    if check is False:
         raise "Invalid image data"
     return data
 
@@ -163,7 +165,7 @@ def _opencv_decoder_(data):
     b = data
     nb = np.asarray(b, dtype=np.uint8)
     data = cv2.imdecode(nb, cv2.IMREAD_COLOR)
-    data = cv2.cvtColor(data, cv2.COLOR_BGR2RGB)
+    data = data[..., ::-1]
     return data
 
 
@@ -171,14 +173,49 @@ def _opencv_encoder_(data, **kargs):
     quality = 90
     if "quality" in kargs:
         quality = kargs["quality"]
-    data = cv2.cvtColor(data, cv2.COLOR_BGR2RGB)
-    check, data = cv2.imencode(
-        ".jpg", data, [int(cv2.IMWRITE_JPEG_QUALITY), quality])  # quality 1-100
-    if check == False:
+    data = data[..., ::-1]
+    check, data = cv2.imencode(".jpg", data, [int(cv2.IMWRITE_JPEG_QUALITY), quality])  # quality 1-100
+    if check is False:
         raise "Invalid image data"
     return data
 
 
+def native_decoder(data):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_encoder(data, *, quality=90, format="jpg"):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_fast_decoder(data):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_fast_encoder(data, *, quality=90, format="jpg"):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_load_image(path):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_save_image(path, data, *, quality=90, format="jpg"):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_fast_load_image(path):
+    raise Exception("You must install aimage-native-ext")
+
+
+def native_fast_save_image(path, data, *, quality=90, format="jpg"):
+    compressed_data = native_module.fast_image_encoder(data, quality, format)
+    fp = _dopen(os.path.expanduser(path), "wb")
+    fp.write(compressed_data)
+    fp.close()
+    #return native_module.fast_image_encoder(path,data,quality,format)
+
+
 def decoder(data):  # @public
     return _opencv_decoder_(data)
 
@@ -189,12 +226,12 @@ def encoder(data, **kargs):  # @public
 
 def load_image(path):  # @public
     img = cv2.imread(path)
-    img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
+    img = img[..., ::-1]
     return img
 
 
 def save_image(path, data, *, quality=90, format="jpg"):  # @public
-    data = cv2.cvtColor(data, cv2.COLOR_RGB2BGR)
+    data = data[..., ::-1]
     return cv2.imwrite(path, data, [cv2.IMWRITE_JPEG_QUALITY, quality])
 
 
@@ -203,12 +240,10 @@ def load(path):  # @public
     if t == "image":
         img = cv2.imread(path, 3)
         if img is None:
-            print(CRED, "\n\nInvalid image file or invalid path. \"%s\"\n\n" %
-                  (path,), CRESET)
+            print(CRED, "\n\nInvalid image file or invalid path. \"%s\"\n\n" % (path, ), CRESET)
             raise "Invalid file or invalid path."
-        return cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
-    print(CRED, "\n\nInvalid image file or invalid path. \"%s\"\n\n" %
-          (path,), CRESET)
+        return img[..., ::-1]
+    print(CRED, "\n\nInvalid image file or invalid path. \"%s\"\n\n" % (path, ), CRESET)
     return None
 
 
@@ -299,14 +334,12 @@ def draw_box(image, box, color, caption=None, fontScale=0.5):  # @public
             raise "Invalid shape."
     elif type(box) == list:
         if len(box) == 2:
-            box = np.array([box[0][0], box[0][1], box[1]
-                            [0], box[1][1]], np.int32)
+            box = np.array([box[0][0], box[0][1], box[1][0], box[1][1]], np.int32)
         else:
             box = np.array([box[0], box[1], box[2], box[3]], np.int32)
     elif type(box) == tuple:
         if len(box) == 2:
-            box = np.array([box[0][0], box[0][1], box[1]
-                            [0], box[1][1]], np.int32)
+            box = np.array([box[0][0], box[0][1], box[1][0], box[1][1]], np.int32)
         else:
             box = np.array([box[0], box[1], box[2], box[3]], np.int32)
     else:
@@ -321,9 +354,7 @@ def draw_box(image, box, color, caption=None, fontScale=0.5):  # @public
     cr = (int(color[0]), int(color[1]), int(color[2]))
     cv2.rectangle(image, c1, c2, cr, box_thick)
     if caption:
-        t_size = cv2.getTextSize(
-            caption, 0, fontScale, thickness=box_thick // 2)[0]
+        t_size = cv2.getTextSize(caption, 0, fontScale, thickness=box_thick // 2)[0]
         c3 = (int(c1[0] + t_size[0]), int(c1[1] - t_size[1] - 3))
         cv2.rectangle(image, c1, c3, cr, -1)  # filled
-        cv2.putText(image, caption, (int(box[0]), int(
-            box[1]) - 2), cv2.FONT_HERSHEY_SIMPLEX, fontScale, (0, 0, 0), box_thick // 2, lineType=cv2.LINE_AA)
+        cv2.putText(image, caption, (int(box[0]), int(box[1]) - 2), cv2.FONT_HERSHEY_SIMPLEX, fontScale, (0, 0, 0), box_thick // 2, lineType=cv2.LINE_AA)
